@@ -24,7 +24,7 @@ use vars qw(@ISA @EXPORT_OK);
 @EXPORT_OK = qw(ExcelFmt LocaltimeExcel ExcelLocaltime
   col2int int2col sheetRef xls2csv);
 
-our $VERSION = '0.49';
+our $VERSION = '0.57';
 
 my $qrNUMBER = qr/(^[+-]?\d+(\.\d+)?$)|(^[+-]?\d+\.?(\d*)[eE][+-](\d+))$/;
 
@@ -68,7 +68,7 @@ my $qrNUMBER = qr/(^[+-]?\d+(\.\d+)?$)|(^[+-]?\d+\.?(\d*)[eE][+-](\d+))$/;
 #
 sub ExcelFmt {
 
-    my ( $format_str, $number, $is_1904, $number_type, $want_color ) = @_;
+    my ( $format_str, $number, $is_1904, $number_type, $want_subformats ) = @_;
 
     # Return text strings without further formatting.
     return $number unless $number =~ $qrNUMBER;
@@ -126,12 +126,12 @@ sub ExcelFmt {
         }
 
         # Convert upper case OpenOffice.org date/time formats to lowercase..
-        $char = lc($char) if $char =~/[DMYHS]/;
+        $char = lc($char) if $char =~ /[DMYHS]/;
 
         $formats[$section] .= $char;
     }
 
-    # Select the appropriate format from the 4 4 possible sub-sections:
+    # Select the appropriate format from the 4 possible sub-sections:
     # positive numbers, negative numbers, zero values, and text.
     # We ignore the Text section since non-numeric values are returned
     # unformatted at the start of the function.
@@ -151,14 +151,14 @@ sub ExcelFmt {
     }
     elsif ( @formats == 3 ) {
         if ( $number == 0 ) {
-            $section = 3;
-        }
-        elsif ( $number < 0 ) {
             $section = 2;
         }
-        else {
+        elsif ( $number < 0 ) {
             $section = 1;
         }
+        else {
+            $section = 0;
+        }
     }
     else {
         $section = 0;
@@ -184,6 +184,12 @@ sub ExcelFmt {
         $color = $1;
     }
 
+    # Remove the locale, such as [$-409], from the format string.
+    my $locale = '';
+    if ( $format =~ s/^(\[\$-\d+\])// ) {
+        $locale = $1;
+    }
+
     # Remove leading # from '# ?/?', '# ??/??' fraction formats.
     $format =~ s{# \?}{?}g;
 
@@ -203,6 +209,7 @@ sub ExcelFmt {
     my $is_currency = 0;      # Number format is a currency.
     my $is_percent  = 0;      # Number format is a percentage.
     my $is_12_hour  = 0;      # Time format is using 12 hour clock.
+    my $seen_dot    = 0;      # Treat only the first "." as the decimal point.
 
     # Parse the format.
   PARSER:
@@ -210,7 +217,12 @@ sub ExcelFmt {
         $start_pos = $pos;
         my $char = substr( $format, $pos, 1 );
 
-        if ( $char !~ /[#0\+\-\.\?eE\,\%]/ ) {
+        # Ignore control format characters such as '#0+-.?eE,%'. However,
+        # only ignore '.' if it is the first one encountered. RT 45502.
+        if ( ( !$seen_dot && $char !~ /[#0\+\-\.\?eE\,\%]/ )
+            || $char !~ /[#0\+\-\?eE\,\%]/ )
+        {
+
             if ( $token_start != -1 ) {
                 push @placeholders,
                   [
@@ -241,7 +253,8 @@ sub ExcelFmt {
         }
 
         if (   ( defined($double_quote) and ($double_quote) )
-            or ( defined($single_quote) and ($single_quote) ) )
+            or ( defined($single_quote) and ($single_quote) )
+            or ( $seen_dot && $char eq '.' ) )
         {
             $single_quote = 0;
             if (
@@ -359,6 +372,7 @@ sub ExcelFmt {
                 elsif ( substr( $format, $pos, 1 ) eq '.' ) {
                     push @placeholders,
                       [ substr( $format, $pos, 1 ), length($token), 1 ];
+                    $seen_dot = 1;
                 }
                 elsif ( substr( $format, $pos, 1 ) eq ',' ) {
                     $comma_count++;
@@ -525,15 +539,26 @@ sub ExcelFmt {
     #
     if ( ( $format_mode eq 'date' ) && ( $number =~ $qrNUMBER ) ) {
 
+        # The maximum allowable date in Excel is 9999-12-31T23:59:59.000 which
+        # equates to 2958465.999+ in the 1900 epoch and 2957003.999+ in the
+        # 1904 epoch. We use 0 as the minimum in both epochs. The 1904 system
+        # actually supports negative numbers but that isn't worth the effort.
+        my $min_date = 0;
+        my $max_date = 2958466;
+        $max_date = 2957004 if $is_1904;
+
+        if ( $number < $min_date || $number >= $max_date ) {
+            return $number;    # Return unformatted number.
+        }
+
         # Process date formats.
         my @time = ExcelLocaltime( $number, $is_1904 );
 
         #    0     1     2      3     4       5      6      7
         my ( $sec, $min, $hour, $day, $month, $year, $wday, $msec ) = @time;
 
-        $month++;       # localtime() zero indexed month.
-        $year += 1900;  # localtime() year.
-
+        $month++;              # localtime() zero indexed month.
+        $year += 1900;         # localtime() year.
 
         my @full_month_name = qw(
           None January February March April May June July
@@ -543,10 +568,10 @@ sub ExcelFmt {
           None Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
         );
         my @full_day_name = qw(
-          Monday Tuesday Wednesday Thursday Friday Saturday Sunday
+          Sunday Monday Tuesday Wednesday Thursday Friday Saturday
         );
         my @short_day_name = qw(
-          Mon Tue Wed Thu Fri Sat Sun
+          Sun Mon Tue Wed Thu Fri Sat
         );
 
         # Replace the placeholders in the template such as yyyy mm dd with
@@ -602,13 +627,13 @@ sub ExcelFmt {
             }
             elsif ( $placeholder->[0] eq 'dddd' ) {
 
-                # Full day name. 1 -> Wednesday (for example.)
-                $replacement = $full_day_name[$msec];
+                # Full day name. Wednesday (for example.)
+                $replacement = $full_day_name[$wday];
             }
             elsif ( $placeholder->[0] eq 'ddd' ) {
 
-                # Short day name. 1 -> Wednesday (for example.)
-                $replacement = $short_day_name[$msec];
+                # Short day name. Wed (for example.)
+                $replacement = $short_day_name[$wday];
             }
             elsif ( $placeholder->[0] eq 'dd' ) {
 
@@ -688,16 +713,16 @@ sub ExcelFmt {
                   sprintf( '%d', ( int($number) * 24 + $hour ) * 60 + $min );
             }
             elsif ( $placeholder->[0] eq 'ge' ) {
-
-                # NENGO (Japanese)
-                $replacement =
-                  Spreadsheet::ParseExcel::FmtJapan::CnvNengo( 1, @time );
+                require Spreadsheet::ParseExcel::FmtJapan;
+                # Japanese Nengo (aka Gengo) in initialism (abbr. name)
+                 $replacement =
+                  Spreadsheet::ParseExcel::FmtJapan::CnvNengo( abbr_name => @time );
             }
             elsif ( $placeholder->[0] eq 'ggge' ) {
-
-                # NENGO (Japanese)
-                $replacement =
-                  Spreadsheet::ParseExcel::FmtJapan::CnvNengo( 2, @time );
+                require Spreadsheet::ParseExcel::FmtJapan;
+                # Japanese Nengo (aka Gengo) in Kanji (full name)
+                 $replacement =
+                  Spreadsheet::ParseExcel::FmtJapan::CnvNengo( name => @time );
             }
             elsif ( $placeholder->[0] eq '@' ) {
 
@@ -764,14 +789,22 @@ sub ExcelFmt {
             }
             else {
                 if ($is_decimal) {
-                    $number_result = sprintf(
-                        (
-                            defined($after_decimal)
-                            ? "%0${str_length}.${after_decimal}f"
-                            : "%0${str_length}f"
-                        ),
-                        $data
-                    );
+
+                    if ( defined $after_decimal ) {
+                        $number_result =
+                          sprintf "%0${str_length}.${after_decimal}f", $data;
+                    }
+                    else {
+                        $number_result = sprintf "%0${str_length}f", $data;
+                    }
+
+                    # Fix for Perl and sprintf not rounding up like Excel.
+                    # http://rt.cpan.org/Public/Bug/Display.html?id=45626
+                    if ( $data =~ /^${number_result}5/ ) {
+                        $number_result =
+                          sprintf "%0${str_length}.${after_decimal}f",
+                          $data . '1';
+                    }
                 }
                 else {
                     $number_result = sprintf( "%0${str_length}.0f", $data );
@@ -913,10 +946,9 @@ sub ExcelFmt {
     $result =~ s/^\$\-/\-\$/;
     $result =~ s/^\$ \-/\-\$ /;
 
-
-    # Return color string if required.
-    if ($want_color) {
-        return ( $result, $color );
+    # Return color and locale strings if required.
+    if ($want_subformats) {
+        return ( $result, $color, $locale );
     }
     else {
         return $result;
@@ -1025,7 +1057,8 @@ sub LeapYear {
 # LocaltimeExcel (for Spreadsheet::ParseExcel::Utility)
 #------------------------------------------------------------------------------
 sub LocaltimeExcel {
-    my ( $iSec, $iMin, $iHour, $iDay, $iMon, $iYear, $iMSec, $flg1904 ) = @_;
+    my ( $iSec, $iMin, $iHour, $iDay, $iMon, $iYear, $iwDay, $iMSec, $flg1904 )
+      = @_;
 
     #0. Init
     $iMon++;
@@ -1084,6 +1117,7 @@ sub LocaltimeExcel {
 # ExcelLocaltime (for Spreadsheet::ParseExcel::Utility)
 #------------------------------------------------------------------------------
 sub ExcelLocaltime {
+
     my ( $dObj, $flg1904 ) = @_;
     my ( $iSec, $iMin, $iHour, $iDay, $iMon, $iYear, $iwDay, $iMSec );
     my ( $iDt, $iTime, $iYDays );
@@ -1110,7 +1144,8 @@ sub ExcelLocaltime {
           (      ( ( $iYear % 4 ) == 0 )
               && ( ( $iYear % 100 ) || ( $iYear % 400 ) == 0 ) ) ? 366 : 365;
     }
-    $iYear -= 1900;
+    $iYear -= 1900;       # Localtime year is relative to 1900.
+
     for ( $iMon = 1 ; $iMon < 12 ; $iMon++ ) {
         my $iMD;
         if (   $iMon == 1
@@ -1133,6 +1168,8 @@ sub ExcelLocaltime {
         $iDt -= $iMD;
     }
 
+    $iMon -= 1;    # Localtime month is 0 based.
+
     #2. Calc Time
     $iDay = $iDt;
     $iTime += ( 0.0005 / 86400.0 );
@@ -1148,7 +1185,7 @@ sub ExcelLocaltime {
     $iTime *= 1000.0;
     $iMSec = int($iTime);
 
-    return ( $iSec, $iMin, $iHour, $iDay, $iMon - 1, $iYear, $iwDay, $iMSec );
+    return ( $iSec, $iMin, $iHour, $iDay, $iMon, $iYear, $iwDay, $iMSec );
 }
 
 # -----------------------------------------------------------------------------
@@ -1193,7 +1230,7 @@ sub int2col {
         $val = int( $val / 26 ) - 1;
     } while ( $val >= 0 );
 
-    return reverse $out;
+    return scalar reverse $out;
 }
 
 # -----------------------------------------------------------------------------
@@ -1233,12 +1270,11 @@ sub sheetRef {
 #
 sub xls2csv {
     my ( $filename, $regions, $rotate ) = @_;
-    my $sheet  = 0;
+    my $sheet = 0;
 
     # We need Text::CSV_XS for proper CSV handling.
     require Text::CSV_XS;
 
-
     # extract any sheet number from the region string
     $regions =~ m/^(\d+)-(.*)/;
 
@@ -1287,7 +1323,7 @@ sub xls2csv {
     my $oWkS = $oBook->{Worksheet}[$sheet];
 
     # now check that the region exists in the file
-    # if not trucnate to the possible region
+    # if not truncate to the possible region
     # output a warning msg
     if ( $start[1] < $oWkS->{MinCol} ) {
         print STDERR int2col( $start[1] )
@@ -1335,7 +1371,7 @@ sub xls2csv {
                 my $cell = $oWkS->{Cells}[$y][$x];
 
                 my $value;
-                if (defined $cell) {
+                if ( defined $cell ) {
                     $value .= $cell->value();
                 }
                 else {
@@ -1353,7 +1389,7 @@ sub xls2csv {
                 my $cell = $oWkS->{Cells}[$y][$x];
 
                 my $value;
-                if (defined $cell) {
+                if ( defined $cell ) {
                     $value .= $cell->value();
                 }
                 else {
@@ -1371,11 +1407,10 @@ sub xls2csv {
     my $output = "";
 
     for my $row (@cell_data) {
-        $csv->combine( @$row );
+        $csv->combine(@$row);
         $output .= $csv->string();
     }
 
-
     return $output;
 }
 
@@ -1560,7 +1595,7 @@ Original author: Kawai Takanori kwitknr@cpan.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 2009 John McNamara
+Copyright (c) 2009-2010 John McNamara
 
 Copyright (c) 2006-2008 Gabor Szabo
 
